@@ -15,9 +15,8 @@
 #
 # You should have received a copy of the GNU Lesser General Public License along
 # with yams. If not, see <http://www.gnu.org/licenses/>.
-"""Classes to define generic Entities/Relations schemas.
+"""Classes to define generic Entities/Relations schemas."""
 
-"""
 __docformat__ = "restructuredtext en"
 
 import warnings
@@ -25,7 +24,7 @@ from copy import deepcopy
 from decimal import Decimal
 
 from logilab.common import attrdict
-from logilab.common.decorators import cached
+from logilab.common.decorators import cached, clear_cache
 from logilab.common.compat import sorted
 from logilab.common.interface import implements
 from logilab.common.deprecation import deprecated
@@ -187,10 +186,27 @@ class EntitySchema(PermissionMixIn, ERSchema):
             self._specialized_by = rdef.specialized_by
             self.final = self.type in BASE_TYPES
             self.permissions = rdef.__permissions__.copy()
-        else:
+            self._unique_together = getattr(rdef, '__unique_together__', [])
+        else: # this happens during deep copy (cf. ERSchema.__deepcopy__)
             self._specialized_type = None
             self._specialized_by = []
 
+    def check_unique_together(self):
+        errors = []
+        for unique_together in self._unique_together:
+            for name in unique_together:
+                try:
+                    rschema = self.rdef(name)
+                except KeyError:
+                    errors.append('no such attribute or relation %s' % name)
+                else:
+                    if not (rschema.final or rschema.rtype.inlined):
+                        errors.append('%s is not an attribute or an inlined '
+                                      'relation' % name)
+        if errors:
+            msg = 'invalid __unique_together__ specification for %s: %s' % (self, ', '.join(errors))
+            raise BadSchemaDefinition(msg)
+
     def __repr__(self):
         return '<%s %s - %s>' % (self.type,
                                  [rs.type for rs in self.subject_relations()],
@@ -211,6 +227,8 @@ class EntitySchema(PermissionMixIn, ERSchema):
     def add_subject_relation(self, rschema):
         """register the relation schema as possible subject relation"""
         self.subjrels[rschema] = rschema
+        clear_cache(self, 'ordered_relations')
+        clear_cache(self, 'meta_attributes')
 
     def add_object_relation(self, rschema):
         """register the relation schema as possible object relation"""
@@ -219,6 +237,8 @@ class EntitySchema(PermissionMixIn, ERSchema):
     def del_subject_relation(self, rtype):
         try:
             del self.subjrels[rtype]
+            clear_cache(self, 'ordered_relations')
+            clear_cache(self, 'meta_attributes')
         except KeyError:
             pass
 
@@ -528,7 +548,7 @@ class EntitySchema(PermissionMixIn, ERSchema):
             if not aschema.check_value(value):
                 errors[qname] = _('incorrect value (%(value)s) for type "%(type)s"') % {
                     'value':value, 'type': _(aschema.type)}
-                if isinstance(value, str):
+                if isinstance(value, str) and aschema == 'String':
                     errors[qname] += '; you might want to try unicode'
                 continue
             # ensure value has the correct python type
@@ -797,9 +817,9 @@ class RelationSchema(ERSchema):
         self.rdefs[key] = rdef = RelationDefinitionSchema(subject, self, object)
         for prop, default in rdef.rproperties().iteritems():
             rdefval = getattr(buildrdef, prop, MARKER)
-            if rdefval is MARKER and prop == 'permissions':
-                rdefval = buildrdef.get_permissions(self.final).copy()
             if rdefval is MARKER:
+                if prop == 'permissions':
+                    rdefval = default = buildrdef.get_permissions(self.final).copy()
                 if prop == 'cardinality':
                     default = (object in BASE_TYPES) and '?1' or '**'
             else:
@@ -979,9 +999,8 @@ class RelationDefinitionSchema(PermissionMixIn):
         return propdefs
 
     def rproperties(self):
-        """return a dictionary mapping property name to its definition for each
-        allowable properties when the relation has `desttype` as target entity's
-        type
+        """same as .rproperty_defs class method, but for instances (hence
+        destination is known to be self.object).
         """
         return self.rproperty_defs(self.object)
 
