@@ -110,6 +110,10 @@ def eschema2sql(dbhelper, eschema, skip_relations=(), prefix=''):
         rschema, attrschema = attrs[i]
         if attrschema is None or eschema.rdef(rschema).indexed:
             w(dbhelper.sql_create_index(table, prefix + rschema.type))
+    for unique_together in eschema._unique_together:
+        cols  = ['%s%s' % (prefix, col) for col in unique_together]
+        sql = dbhelper.sql_create_multicol_unique_index(table, cols)
+        w(sql)
     w('')
     return '\n'.join(output)
 
@@ -161,9 +165,6 @@ def type_from_constraints(dbhelper, etype, constraints, creating=True):
         sqltype += ' UNIQUE'
     return sqltype
 
-# backward compat
-_type_from_constraints = type_from_constraints
-
 
 _SQL_SCHEMA = """
 CREATE TABLE %(table)s (
